package gr.vidavo;

import gr.vidavo.tools.DateTimeTools;
import gr.vidavo.tools.MathTools;
import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.nio.DoubleBuffer;
import javax.microedition.io.StreamConnection;
import javax.microedition.io.Connector;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import java.io.IOException;


public class ECGApp implements Runnable {

	String serverURL;
	String user;
	private Thread ecgThread = null;

	public ECGApp(String serverURLarg,String username) {	
		user=username;
		serverURL = serverURLarg;

		if (ecgThread == null) {
			ecgThread = new Thread(this,"ECGThread");
			ecgThread.start();
		}

	}

	public void run() {


		createFile();
		launchECGProgress();
		connect();

		stop();
	}

	//terminate current thread
	public void stop() {

		ecgThread = null;

		if (ecgThread != null) {
			ecgThread.interrupt();
		}

	}


	//sampling frequency
	int fs = 250;

	//Period Ts of sampling
	float Ts = 1f/fs;

	//Calibration time for each phase
	long calibrationTime = 3500;


	String header1 = "[TRANSMISSION HEADER]";
	String version = "Version=3.5.6.9";
	String sampleRate = "SampleRate=" + fs;
	String dynamicRange = "DynamicRange=6";
	String eventsNumber = "EventsNumber=";
	String samplesNumberInEvent = "SamplesNumberInEvent=";
	String postEventInSec = "PostEventInSec=0";
	String eventDate = "EventDate=";
	String eventTime = "EventTime=";
	String dataTimeOfRecording = "DataTimeOfRecording=No";
	String eventAuto = "EventAuto=No";
	String monitorSerialNumber = "MonitorSerialNumber=0"; 

	File ecgFile;
	ECGTestProgress ecgProgress;
	static boolean progressIsOpen = false;
	FileOutputStream fos;
	static StreamConnection sc;
	static DataOutputStream dos;	
	static DataInputStream dis;
	byte[] initCommand1 = {0x20,0x20};
	byte[] initCommand2 = {0x51,0x51};
	byte[] pingCommand = {0x45,0x45,0x00};
	byte[] successPingResponse = {(byte)0xfc,0x45,0x01,(byte)0xfd,0x45,0x00,0x00,0x00,0x00};
	byte[] oneLeadStartCommand = {0x40,0x40,0x00};
	byte[] twelveLeadPhase1StartCommand = {0x40,0x40,0x01};
	byte[] twelveLeadPhase2StartCommand = {0x40,0x40,0x02};
	byte[] twelveLeadPhase3StartCommand = {0x40,0x40,0x03};
	byte[] startResponse = new byte[5];
	byte[] startSuccessResponse = {-4,0x40,0x01,-3,0x40};
	byte[] stopCommand = {0x41,0x41};
	byte[] closeCommand = {0x52,0x52};
	byte[] initResponse1=new byte[30];
	byte[] pingResponse = new byte[9];
	boolean oneLead;
	boolean twelveLead;
	String dateecg=null;

	public void createFile () {
		dateecg=DateTimeTools.getDateTimeString2();
		//we create a unique File with date and time signature with a ecg extension
		ecgFile = new File(Consts.ECG_FOLDER +"\\ECG_" +dateecg +".ecg");		
	}

	/*
	 * a simple method, just to display the progress of the ECG
	 */
	public void launchECGProgress() {

		ecgProgress = new ECGTestProgress();
		ecgProgress.setVisible(true);

	}


	public void connect() {	

		try {
			sc = (StreamConnection)Connector.open("btspp://" +serverURL +":1",3);

			 dos = new DataOutputStream(sc.openOutputStream());	
			dis = new DataInputStream(sc.openDataInputStream());

			//once we re connected, change the status of the progress bar and the label
			ecgProgress.getJProgressBar().setIndeterminate(false);
			ecgProgress.jLabel.setText("");

			dos.write(initCommand1);
			dos.flush();
			dis.readFully(initResponse1);


			//here we can understand if the user will initiate a 12Lead test or a simple 1Lead
			dos.write(initCommand2);
			dos.flush();
			int[] resp=new int[7];
			for(int i=0;i<7;i++){
	            resp[i]=dis.read();
	            }
	            
	            
	            int f=resp[5]&0xFF;
	            int f1=(f>>2 & 0x7);
	            if(f1==0)
	            {
	               
	                oneLead=true;
					ecgProgress.jLabel.setText(" 1-");
	            }
	            else if(f1==1)
	            {
	                
	                twelveLead=true;
					ecgProgress.jLabel.setText(" 12-");
	            }

			

			dos.write(pingCommand);
			dos.flush();
			byte[] resp1=new byte[9];
			for(int i=0;i<9;i++){
	            resp1[i]=(byte)dis.read();
	            }		

		
			while (!java.util.Arrays.equals( resp1, successPingResponse)) {			
				try{Thread.sleep(1000);}catch(InterruptedException e) {}
				dos.write(pingCommand);
				dos.flush();
				for(int i=0;i<9;i++){
	                resp1[i]=(byte)dis.read();
	                  }			

			}

			if (oneLead) {
				this.oneLeadTest();
			}
			else if (twelveLead) {
				twelveLeadTest();
			}
			else {
			}

		}
		catch (IOException e1) {
			try{Thread.sleep(1000);}catch(InterruptedException e) {}
			if (progressIsOpen) {
				connect();
			}
		}catch(Exception ex){
			ecgProgress.dispose();
			progressIsOpen = false;
			JOptionPane.showMessageDialog(new JFrame(),"   .\n  ->.","",JOptionPane.WARNING_MESSAGE);
			stop();
		}		
	}
	


	public void oneLeadTest() {
		byte[] stResponse=new byte[5];
		
		try {
			dos.write(oneLeadStartCommand);
			dos.flush();
			dis.readFully(stResponse);
			if (java.util.Arrays.equals( startSuccessResponse, stResponse)) {
				ecgProgress.getJProgressBar().setIndeterminate(true);
			}
			else 
			{
			}

			//The ECG duration, in ms, is defined here. 
			//3500 calibration time is used by SelfCheckECG
			long duration = 34000;

			//maxSamples is depended on the duration and the sampling rate.
			int maxSamples = (int)((duration/1000)*(fs+50));
			DoubleBuffer ecgValues = DoubleBuffer.allocate(maxSamples);

			long startTime = System.currentTimeMillis();

			while (System.currentTimeMillis() - startTime < calibrationTime + duration) {				
				ecgValues.put(read2ByteValue());
			}

			int totalSamples = ecgValues.position();	
			byte[] ack=new byte[3];

			
			//we are waiting for the 0xfc4101 pattern
			dos.write(stopCommand);
            dos.flush();
			dis.read(ack);
			dos.write(stopCommand);
			dos.flush();
			dos.write(closeCommand);
			dos.flush();
			dis.close();
			dos.close();
			sc.close();
			
			
			ecgProgress.dispose();
			progressIsOpen = false;
			

		
			double[] ecgValuesArray = MathTools.offsetData(ecgValues, totalSamples);
			
			String fileData = 
				header1 +"\r\n" 
				+version +"\r\n" 
				+"DeviceSoftwareCode=" +" " +"\r\n" 
				+sampleRate +"\r\n" 
				+dynamicRange +"\r\n"
				+eventsNumber +1 +"\r\n" 
				+samplesNumberInEvent +ecgValues.position() +"\r\n"
				+postEventInSec +"\r\n"
				+"[HEADER Event1]" +"\r\n"
				+eventDate +DateTimeTools.getDateString() +"\r\n"
				+eventTime +DateTimeTools.getTimeString() +"\r\n"
				+dataTimeOfRecording +"\r\n"
				+eventAuto +"\r\n"
				+monitorSerialNumber +"\r\n"
				+"[ECG Event1]" +"\r\n";			

			//write the header info
			try {
				fos = new FileOutputStream(ecgFile);
			}
			catch (java.io.FileNotFoundException e) {}

			fos.write(fileData.getBytes());

		
			for (int i=0; i<totalSamples; i++) {				
				fos.write(((int)ecgValuesArray[i] +"\r\n").getBytes());
			}

			
			fos.close();
			

			//construct the time domain array
			double[] time = new double[totalSamples];
			for (int i=0; i<totalSamples; i++) {
				time[i] = i*Ts;
			}

		
			savelocal sl=new savelocal();
			sl.savelocalecg(DateTimeTools.getDateTimeString1(), "ECG_" +dateecg +".ecg",1);
			
			ECGFileReader.event=sl.getEventId();
			new ECGFileReader(ecgFile.getPath(),user);
		}
		catch (IOException e1) {} 
	}

	public void twelveLeadTest() {

		try {
			
			
			
			dos.write(twelveLeadPhase1StartCommand);
				dos.flush();
			dis.readFully(startResponse);
			if (java.util.Arrays.equals( startSuccessResponse, startResponse)) {
				ecgProgress.getJProgressBar().setIndeterminate(true);
				ecgProgress.jLabel.setText(" ");
			}
			else 
			{
			}
			ecgProgress.jLabel.setText(" 1, V1-V2");


			//we define the duration of the 3 different ECG phases 
			long phase1Duration = 12000, phase2Duration = 5000,phase3Duration = 5000;

			//maxSamples is depended on the duration and the sampling rate.
			int maxSamplesPhase1 = (int)((phase1Duration/1000)*(fs+50));
			DoubleBuffer phase1Value1 = DoubleBuffer.allocate(maxSamplesPhase1);
			DoubleBuffer phase1Value2 = DoubleBuffer.allocate(maxSamplesPhase1);
			DoubleBuffer phase1Value3 = DoubleBuffer.allocate(maxSamplesPhase1);
			DoubleBuffer phase1Value4 = DoubleBuffer.allocate(maxSamplesPhase1);

			long startTimePhase1 = System.currentTimeMillis();

//			##########Phase1
			while (System.currentTimeMillis() - startTimePhase1 < calibrationTime + phase1Duration) {
				//this inner loop reads and stores the four different values from the ECG - phase 1.

				phase1Value1.put(read2ByteValue());
				phase1Value2.put(read2ByteValue());
				phase1Value3.put(read2ByteValue());
				phase1Value4.put(read2ByteValue());



			}

			int totalSamplesPhase1 = phase1Value1.position();


			//we send a stop command and wait for the pattern 0xfc4101
			dos.write(stopCommand);
			dos.flush();
			for (int i=0;i<5000;i++) {
				if (dis.readUnsignedByte() == 0xfc) {
					if (dis.readUnsignedByte() == 0x41 ) {
						if (dis.readUnsignedByte() == 0x01) {
							break;
						}
					}
				}
			}

			//display and beep before phase 2 begins
			ecgProgress.jLabel.setText(" 2, V3-V4");
			ecgProgress.getJProgressBar().setIndeterminate(false);
			java.awt.Toolkit.getDefaultToolkit().beep();
			JOptionPane.showMessageDialog(null, "     ", "ECG ", JOptionPane.PLAIN_MESSAGE);

			try{Thread.sleep(2000);}catch(InterruptedException e) {}
			dos.write(pingCommand);
			dos.flush();
			for(int i=0;i<pingResponse.length;i++){
                pingResponse[i]=(byte)dis.read();
            }

			while (!java.util.Arrays.equals( pingResponse, successPingResponse)) {
				try{Thread.sleep(1000);}catch(InterruptedException e) {}
				dos.write(pingCommand);
				for(int i=0;i<pingResponse.length;i++){
                    pingResponse[i]=(byte)dis.read();
                     }				

			}

			dos.write(twelveLeadPhase2StartCommand);
			dos.flush();
			dis.readFully(startResponse);
			if (!java.util.Arrays.equals( startSuccessResponse, startResponse)) {
			}

			ecgProgress.getJProgressBar().setIndeterminate(true);
			ecgProgress.jLabel.setText(" ");

			//maxSamples is depended on the duration and the sampling rate.
			int maxSamplesPhase2 = (int)((phase2Duration/1000)*(fs+50));
			DoubleBuffer phase2Value1 = DoubleBuffer.allocate(maxSamplesPhase2);
			DoubleBuffer phase2Value2 = DoubleBuffer.allocate(maxSamplesPhase2);
			DoubleBuffer phase2Value3 = DoubleBuffer.allocate(maxSamplesPhase2);
			DoubleBuffer phase2Value4 = DoubleBuffer.allocate(maxSamplesPhase2);

//			##########	Phase2
			long startTimePhase2 = System.currentTimeMillis();
			while (System.currentTimeMillis() - startTimePhase2 < calibrationTime + phase2Duration) {
				//this inner loop reads and stores the four different values from the ECG

				phase2Value1.put(read2ByteValue());
				phase2Value2.put(read2ByteValue());
				phase2Value3.put(read2ByteValue());
				phase2Value4.put(read2ByteValue());



			}

			int totalSamplesPhase2 = phase2Value1.position();



//			we send a stop command and wait for the pattern 0xfc4101
			dos.write(stopCommand);
			dos.flush();
			for (int i=0;i<5000;i++) {
				if (dis.readUnsignedByte() == 0xfc) {
					if (dis.readUnsignedByte() == 0x41 ) {
						if (dis.readUnsignedByte() == 0x01) {
							break;
						}
					}
				}
			}

			//display and beep before phase 2 begins
			ecgProgress.jLabel.setText(" 3, V5-V6");
			ecgProgress.getJProgressBar().setIndeterminate(false);
			java.awt.Toolkit.getDefaultToolkit().beep();
			JOptionPane.showMessageDialog(null, "     ", "ECG ", JOptionPane.PLAIN_MESSAGE);

			try{Thread.sleep(2000);}catch(InterruptedException e) {}
			dos.write(pingCommand);
			dos.flush();
			
			for(int i=0;i<pingResponse.length;i++){
                pingResponse[i]=(byte)dis.read();
            }


			while (!java.util.Arrays.equals( pingResponse, successPingResponse)) {
				try{Thread.sleep(1000);}catch(InterruptedException e) {}
				dos.write(pingCommand);
				dos.flush();
				
				for(int i=0;i<pingResponse.length;i++){
	                pingResponse[i]=(byte)dis.read();
	            }			

			}

			dos.write(twelveLeadPhase3StartCommand);
			dos.flush();
			
			dis.readFully(startResponse);
			if (!java.util.Arrays.equals( startSuccessResponse, startResponse)) {
			}
			
			ecgProgress.getJProgressBar().setIndeterminate(true);
			ecgProgress.jLabel.setText(" ");


			//maxSamples is depended on the duration and the sampling rate.
			int maxSamplesPhase3 = (int)((phase3Duration/1000)*(fs+50));
			DoubleBuffer phase3Value1 = DoubleBuffer.allocate(maxSamplesPhase3);
			DoubleBuffer phase3Value2 = DoubleBuffer.allocate(maxSamplesPhase3);
			DoubleBuffer phase3Value3 = DoubleBuffer.allocate(maxSamplesPhase3);
			DoubleBuffer phase3Value4 = DoubleBuffer.allocate(maxSamplesPhase3);



			long startTimePhase3 = System.currentTimeMillis();
			while (System.currentTimeMillis() - startTimePhase3 < calibrationTime + phase3Duration) {

				phase3Value1.put(read2ByteValue());
				phase3Value2.put(read2ByteValue());
				phase3Value3.put(read2ByteValue());
				phase3Value4.put(read2ByteValue());

			}

			int totalSamplesPhase3 = phase3Value1.position();



			//we send a stop command and wait for the pattern 0xfc4101
			byte[] ack=new byte[3];
			dos.write(stopCommand);
			dos.flush();
			dis.readFully(ack);
			dos.write(closeCommand);
			dos.flush();
			
			
			//test is complete. close the progress
			ecgProgress.dispose();
			progressIsOpen = false;
			
			double[] da = new double[totalSamplesPhase1];
			phase1Value1.position(0);
			phase1Value1.get(da);
			double[] db = new double[totalSamplesPhase1];
			phase1Value2.position(0);
			phase1Value2.get(db);
			double[] dc = new double[totalSamplesPhase1];
			phase1Value3.position(0);
			phase1Value3.get(dc);
			double[] dd = new double[totalSamplesPhase1];
			phase1Value4.position(0);
			phase1Value4.get(dd);
			double[] de = new double[totalSamplesPhase2];
			phase2Value1.position(0);
			phase2Value1.get(de);
			double[] df = new double[totalSamplesPhase2];
			phase2Value2.position(0);
			phase2Value2.get(df);
			double[] dg = new double[totalSamplesPhase2];
			phase2Value3.position(0);
			phase2Value3.get(dg);
			double[] dh = new double[totalSamplesPhase2];
			phase2Value4.position(0);
			phase2Value4.get(dh);
			double[] di = new double[totalSamplesPhase3];
			phase3Value1.position(0);
			phase3Value1.get(di);
			double[] dj = new double[totalSamplesPhase3];
			phase3Value2.position(0);
			phase3Value2.get(dj);
			double[] dl = new double[totalSamplesPhase3];
			phase3Value3.position(0);
			phase3Value3.get(dl);
			double[] dm = new double[totalSamplesPhase3];
			phase3Value4.position(0);
			phase3Value4.get(dm);
			
		

			double[] phase1Value1Array = MathTools.offsetData(phase1Value1, totalSamplesPhase1);
			double[] phase1Value2Array = MathTools.offsetData(phase1Value2, totalSamplesPhase1);
			double[] phase1Value3Array = MathTools.offsetData(phase1Value3, totalSamplesPhase1);
			double[] phase1Value4Array = MathTools.offsetData(phase1Value4, totalSamplesPhase1);
			double[] phase2Value1Array = MathTools.offsetData(phase2Value1, totalSamplesPhase2);
			double[] phase2Value2Array = MathTools.offsetData(phase2Value2, totalSamplesPhase2);
			double[] phase2Value3Array = MathTools.offsetData(phase2Value3, totalSamplesPhase2);
			double[] phase2Value4Array = MathTools.offsetData(phase2Value4, totalSamplesPhase2);
			double[] phase3Value1Array = MathTools.offsetData(phase3Value1, totalSamplesPhase3);
			double[] phase3Value2Array = MathTools.offsetData(phase3Value2, totalSamplesPhase3);
			double[] phase3Value3Array = MathTools.offsetData(phase3Value3, totalSamplesPhase3);
			double[] phase3Value4Array = MathTools.offsetData(phase3Value4, totalSamplesPhase3);


			String fileHeader = 
				header1 +"\r\n" 
				+version +"\r\n" 
				+"DeviceSoftwareCode=" +" " +"\r\n" 
				+sampleRate +"\r\n" 
				+dynamicRange +"\r\n"
				+eventsNumber +12 +"\r\n" 



				+samplesNumberInEvent +totalSamplesPhase1 +"\r\n"
				+postEventInSec +"\r\n"
				+getECGEventHeader(1);			

			String eventHeader2 = getECGEventHeader(2);
			String eventHeader3 = getECGEventHeader(3);
			String eventHeader4 = getECGEventHeader(4);
			String eventHeader5 = getECGEventHeader(5);
			String eventHeader6 = getECGEventHeader(6);
			String eventHeader7 = getECGEventHeader(7);
			String eventHeader8 = getECGEventHeader(8);
			String eventHeader9 = getECGEventHeader(9);
			String eventHeader10 = getECGEventHeader(10);
			String eventHeader11 = getECGEventHeader(11);
			String eventHeader12 = getECGEventHeader(12);

			try {
				fos = new FileOutputStream(ecgFile);

			}
			catch (java.io.FileNotFoundException e) {}

			//write the header info
			fos.write(fileHeader.getBytes());

			//now put all values from the buffer to the file.
			for (int i=0; i<totalSamplesPhase1; i++) {				
				fos.write(((int)phase1Value1Array[i] +"\r\n").getBytes());
			}
			fos.write(eventHeader2.getBytes());

			for (int i=0; i<totalSamplesPhase1; i++) {				
				fos.write(((int)phase1Value2Array[i] +"\r\n").getBytes());
			}
			fos.write(eventHeader3.getBytes());

			for (int i=0; i<totalSamplesPhase1; i++) {				
				fos.write(((int)phase1Value3Array[i] +"\r\n").getBytes());
			}

			fos.write(eventHeader4.getBytes());

			for (int i=0; i<totalSamplesPhase1; i++) {				
				fos.write(((int)phase1Value4Array[i] +"\r\n").getBytes());
			}

			fos.write(eventHeader5.getBytes());

			for (int i=0; i<totalSamplesPhase2; i++) {				
				fos.write(((int)phase2Value1Array[i] +"\r\n").getBytes());
			}

			fos.write(eventHeader6.getBytes());

			for (int i=0; i<totalSamplesPhase2; i++) {				
				fos.write(((int)phase2Value2Array[i] +"\r\n").getBytes());
			}

			fos.write(eventHeader7.getBytes());

			for (int i=0; i<totalSamplesPhase2; i++) {				
				fos.write(((int)phase2Value3Array[i] +"\r\n").getBytes());
			}

			fos.write(eventHeader8.getBytes());

			for (int i=0; i<totalSamplesPhase2; i++) {				
				fos.write(((int)phase2Value4Array[i] +"\r\n").getBytes());
			}

			fos.write(eventHeader9.getBytes());

			for (int i=0; i<totalSamplesPhase3; i++) {				
				fos.write(((int)phase3Value1Array[i] +"\r\n").getBytes());
			}

			fos.write(eventHeader10.getBytes());

			for (int i=0; i<totalSamplesPhase3; i++) {				
				fos.write(((int)phase3Value2Array[i] +"\r\n").getBytes());
			}

			fos.write(eventHeader11.getBytes());

			for (int i=0; i<totalSamplesPhase3; i++) {				
				fos.write(((int)phase3Value3Array[i] +"\r\n").getBytes());
			}

			fos.write(eventHeader12.getBytes());

			for (int i=0; i<totalSamplesPhase3; i++) {				
				fos.write(((int)phase3Value4Array[i] +"\r\n").getBytes());
			}

			fos.close();
			dis.close();
			dos.close();
			sc.close();

			

			//plot the ECG using the ECGFileReader Class.
			savelocal sl=new savelocal();
			sl.savelocalecg(DateTimeTools.getDateTimeString1(), "ECG_" +dateecg +".ecg",12);
			ECGFileReader.event=sl.getEventId();
			new ECGFileReader(ecgFile.getPath(),user);

		}
		catch (IOException e1) {} 
	}


	/*
	 * A function that reads in LITTLE ENDIAN format a 2 byte value, through an open bluetooth RFCOMM connection
	 * If you call this function, the next 2 bytes of the input stream will be read.
	 */
	public short read2ByteValue() {
		int lsb = 0,msb = 0;
		short data=0;
		
		try {
			lsb = (short)dis.read();
			msb = (short)dis.read();
                       

                        if (lsb < 0)
                             lsb = (short)(lsb + 256);
                        if (msb < 0)
                            msb = (short)(msb + 256);
                        data=(short)(lsb+256*msb & 0x3FF);


		}
		catch (Exception e)
         {
			
        }
		
                return data;
	}

	
	public String getECGEventHeader(int eventNumber) {
		String headerData = 
			"[HEADER Event" +eventNumber +"]" +"\r\n"
			+eventDate +DateTimeTools.getDateString() +"\r\n"
			+eventTime +DateTimeTools.getTimeString() +"\r\n"
			+dataTimeOfRecording +"\r\n"
			+eventAuto +"\r\n"
			+monitorSerialNumber +"\r\n"
			+"[ECG Event" +eventNumber +"]" +"\r\n";	
		return headerData;

	}

	public static void main(String[] args) {


	}
}