package oxypro;

import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.OutputStream;
import java.io.InputStream;
import java.nio.ByteBuffer;
import javax.microedition.io.StreamConnection;
import javax.microedition.io.StreamConnectionNotifier;
import javax.microedition.io.Connector;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.bluetooth.ServiceRecord;
import javax.bluetooth.UUID;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.BluetoothStateException;
import java.io.IOException;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Vector;

//import tools.DateTimeTools;

public class OxyProApp implements Runnable{

	private Thread oxyThread = null;
	StreamConnection sc;
	DataOutputStream dos;
	DataInputStream dis;
	static Integer hrMin=250,hrMax=0,hrAvg=0,spo2=0,spo2Min=100,spo2Max=0,spo2Avg=0;
	String serverURL;
	static String deviceIDString;
	String message = "32";  
	//that is the hex 0x20
	int handshake = 32; 
	int finish = 64; 
	String serviceURL = "btspp://localhost:27012f0c68af4fbf8dbe6bbaf7ab651b;name=SPP";

	static boolean progressIsOpen = false;
	static boolean viewerIsOpen = false;

	OxyTestProgress oxyProgress;
	OxyProViewer oxyViewer;
	OxyTest oxyObj;

	public OxyProApp(String serverURLarg) {
		System.out.println("Entering Constructor...");
		serverURL = serverURLarg;

		//construct the local thread
		if (oxyThread == null) {
			oxyThread = new Thread(this,"OxyThread");
			oxyThread.start();
		}

	}

	//this is the code that will run by the thread
	public void run() {

		//while (oxyThread != null) {
		launchProgress();
		createConnection();
		//}

		stop();
	}

	//launches the OxyProApp progress window
	public void launchProgress() {
		oxyProgress = new OxyTestProgress();
		oxyProgress.setVisible(true);
	}

	//initiate a bluetooth connection to the device. if the device isn't ready, just try to reconnect
	public void createConnection() {

		try{
			System.out.println("Creating connection ...");
			System.out.println("Connecting to "+serverURL);
			sc = (StreamConnection)Connector.open("btspp://" +serverURL +":1",3); 

			dos = new DataOutputStream(sc.openOutputStream());
			dis = new DataInputStream(sc.openDataInputStream());

			//once we re connected, change the status of the progress bar and the label
			oxyProgress.getJProgressBar().setIndeterminate(false);
			oxyProgress.getJProgressBar().setValue(100); 
			oxyProgress.jLabel.setText("");

			try{Thread.sleep(1000);}catch(InterruptedException e2) {} 
			oxyProgress.dispose();
			progressIsOpen = false;

			//launch the viewer
			oxyViewer = new OxyProViewer();

			//begin
			startStreaming();
		}

		catch (java.io.IOException e) {
			System.out.println(e.getMessage());
			//JOptionPane.showMessageDialog(null, "Connection Failed", "Goodbye", JOptionPane.PLAIN_MESSAGE);
			System.out.println("Trying again...");
			try{Thread.sleep(2000);}catch(InterruptedException e2) {}

			//if the user is still waiting for the App to connect, then try again.
			if (progressIsOpen) {
				createConnection();
			}
			else {
				stop();
			}
		}

	}

	//terminate current thread
	public void stop() {

		oxyThread = null;

		if (oxyThread != null) {
			oxyThread.interrupt();
		}

	}

	//this method shoulb be called only after the createConnection(). It streams oxypro data
	//and sends them to the viewer

	//TO DO :: a better way to implement streaming between OxyPro and the Viewer would be by using pipes!!
	//and not by statically assign the values from the OxyProApp to the viewer...
	public void startStreaming() {

		//init the OxyTest object
		oxyObj = new OxyTest();
		

		try {
			//try{Thread.sleep(1000);}catch(InterruptedException e) {}
			System.out.println("Sending data");

			byte[] initCommand1 = {0x20,0x20}; 

			//1st response is 30 bytes long - the first 24 bytes are unknown info
			//and the last 6 are the device ID
			byte[] response1 = new byte[24];
			byte[] deviceID = new byte[6];

			dos.write(initCommand1);
			try{Thread.sleep(1000);}catch(InterruptedException e) {}
			dis.read(response1);

			//read the Device ID
			System.out.println("Reading Device ID");
			dis.read(deviceID);
			System.out.println("Success");

			//decode deviceID
			deviceIDString = "";
			for (int i=0; i<deviceID.length; i++) {
				if (deviceID[i] < 10) {
					deviceIDString += "0" +deviceID[i]; 
				}
				else {
					deviceIDString += deviceID[i];
				}
			}

			//store the deviceID to the oxyObject
			oxyObj.setDeviceID(deviceIDString);

			System.out.println("Device ID: " +deviceIDString);

			//2nd response is 7 bytes long
			byte[] initCommand2 = {0x51,0x51}; 
			byte[] response2 = new byte[7];
			dos.write(initCommand2);
			try{Thread.sleep(1000);}catch(InterruptedException e) {}
			dis.read(response2);

			byte[] initCommand3 = {0x40,0x40}; 
			dos.write(initCommand3);
			//first 5 bytes of transmision are bulk.
			byte[] bulk = new byte[5]; 
			dis.read(bulk);

//			//1000 bytes means 1000 seconds of data, pretty big enough!
//			ByteBuffer hr = ByteBuffer.allocate(1000);
//			ByteBuffer o2 = ByteBuffer.allocate(1000);

			Integer hr=0,hrSum=0,spo2Sum=0,seperator=0,count=0;
			//stop test is a static variable that is handled by the viewer
			while (viewerIsOpen) {
				count++;

				//every 2 bytes of data, the oxymeter sends a seperator, a byte which contains:
				//value 0x80 in case of normal operation
				//value 0xa3 when the device is not ready
				//value 0xbf when the there is no finger in the sensor!
				seperator = dis.read();


				//chech the seperator value

				//in case of normal operation...
				if (seperator == 0x80) {
					//read heart rate and show it to viewer
					hr = dis.read();
					oxyViewer.jLabel6.setText(hr.toString());

					//read SPO2 and show it to viewer
					spo2 = dis.read();
					oxyViewer.jLabel7.setText(spo2.toString());

					//increment the sums
					hrSum += hr;
					spo2Sum += spo2;

					if (hr < hrMin) {
						hrMin = hr;
					}
					oxyViewer.jLabel.setText("HR min : " +hrMin.toString());

					if (hr > hrMax) {
						hrMax = hr;
					}
					oxyViewer.jLabel1.setText("HR max : " +hrMax.toString());

					hrAvg = hrSum/count;
					oxyViewer.jLabel2.setText("HR avg : " +hrAvg.toString());

					if (spo2 < spo2Min) {
						spo2Min = spo2;
					}
					oxyViewer.jLabel3.setText("SPO2 min : " +spo2Min.toString());


					if (spo2 > spo2Max) {
						spo2Max = spo2;
					}
					oxyViewer.jLabel4.setText("SPO2 max : " +spo2Max.toString());

					spo2Avg = spo2Sum/count;
					oxyViewer.jLabel5.setText("SPO2 avg : " +spo2Avg.toString());

					//at the end of each loop, update the OxyTest object and set it to the viewer.
					oxyObj.setSpo2Min(String.valueOf(spo2Min));
					oxyObj.setSpo2Max(String.valueOf(spo2Max));
					oxyObj.setSpo2Avg(String.valueOf(spo2Avg));
					oxyObj.setHrMin(String.valueOf(hrMin));
					oxyObj.setHrMax(String.valueOf(hrMax));
					oxyObj.setHrAvg(String.valueOf(hrAvg));

					oxyViewer.oxyObj = oxyObj;

				}
				//in case of 0xa3 seperator
				else if (seperator == 0xa3) {
					//read the next 2 bulk measurments of HR and SPO2
					dis.read();
					dis.read();

					//display zero values to the viewer
					oxyViewer.jLabel6.setText("0");
					oxyViewer.jLabel7.setText("0");

					//display the error to user
					//JOptionPane.showMessageDialog(null, "Device not ready", "Status", JOptionPane.PLAIN_MESSAGE);

					//do nout count this measurment
					count--;
				}

				//in case of 0xbf seperator
				else if (seperator == 0xbf) {
					//read the next 2 bulk measurments of HR and SPO2
					dis.read();
					dis.read();

					//display zero values to the viewer
					oxyViewer.jLabel6.setText("0");
					oxyViewer.jLabel7.setText("0");

					//display the error to user
					//JOptionPane.showMessageDialog(null, "Check finger", "Status", JOptionPane.PLAIN_MESSAGE);

					//do nout count this measurment
					count--;
				}

				//in case of an unknown seperator
				else {
					//read the next 2 bulk measurments of HR and SPO2
					dis.read();
					dis.read();

					//do nout count this measurment
					count--;

				}

			}

			//send stop command {0x52,0x52}
			byte[] stopCommand = {0x52,0x52};
			dos.write(stopCommand);

			dos.close();
			dis.close();
			sc.close();

			System.out.println("Exiting connect method");
		}
		catch (java.io.IOException e) {
			System.out.println(e.getMessage());
			stop();
		}


	}

	public static void main(String[] args) {

		System.out.println("Main method started!");
		//new ConnectTest("0016206c299a"); //my mobile
		//new OxyProApp("00025b00a5a5"); //Oxypro with sticker
		//new OxyProApp("00a09609418f");
		//new OxyProApp("00a09612a946");
		new OxyProApp("00a09609418f"); 
		System.out.println("Exiting main method");

	}

}